package org.dynmap.factions;

import com.massivecraft.factions.entity.Faction;
import com.massivecraft.factions.event.EventFactionsChunksChange;
import com.massivecraft.factions.event.EventFactionsCreate;
import com.massivecraft.factions.event.EventFactionsDisband;
import com.massivecraft.factions.event.EventFactionsMembershipChange;
import com.massivecraft.factions.event.EventFactionsNameChange;
import com.massivecraft.factions.event.EventFactionsWarpAdd;
import com.massivecraft.factions.event.EventFactionsWarpRemove;
import org.bukkit.Bukkit;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.server.PluginEnableEvent;
import org.bukkit.plugin.Plugin;
import org.jetbrains.annotations.NotNull;

public class DynmapFactionsListener implements Listener {

    private final DynmapFactionsPlugin dynmapFactionsPlugin;

    public DynmapFactionsListener(DynmapFactionsPlugin dynmapFactionsPlugin) {
        this.dynmapFactionsPlugin = dynmapFactionsPlugin;
    }

    @EventHandler
    public void onPluginEnable(@NotNull PluginEnableEvent event) {
        Plugin p = event.getPlugin();
        String name = p.getDescription().getName();
        if (name.equals("dynmap") || name.equals("Factions")) {
            if (dynmapFactionsPlugin.dynmap.isEnabled() && dynmapFactionsPlugin.factions.isEnabled()) {
                dynmapFactionsPlugin.activate();
            }
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFPlayerJoin(@NotNull EventFactionsMembershipChange event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.playerSets) {
            Faction f = event.getNewFaction();
            dynmapFactionsPlugin.requestUpdatePlayerSet(f.getId());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionCreate(@NotNull EventFactionsCreate event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.playerSets) {
            dynmapFactionsPlugin.requestUpdatePlayerSet(event.getFactionId());
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionDisband(@NotNull EventFactionsDisband event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.playerSets) {
            Faction f = event.getFaction();
            dynmapFactionsPlugin.requestUpdatePlayerSet(f.getId());
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsNameChange event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsWarpAdd event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsWarpRemove event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsChunksChange event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    private void requestUpdateFactions() {
        if (DynmapFactionsPlugin.instance.pending == null) {
            FactionsUpdate factionsUpdate = new FactionsUpdate(DynmapFactionsPlugin.instance,
                    DynmapFactionsPlugin.instance.getConfig());
            factionsUpdate.runonce = true;
            DynmapFactionsPlugin.instance.pending = factionsUpdate;
            Bukkit.getServer().getScheduler().scheduleSyncDelayedTask(DynmapFactionsPlugin.instance, factionsUpdate, 20);
        }
    }

}
